//+------------------------------------------------------------------+
//|                                                   Timekeeper.mq4 |
//|                                     contactchristinali@gmail.com |
//|                               http//www.wix.com/wiseea/wise-ea#! | 
//+------------------------------------------------------------------+
#property copyright     "Programmed by Christina Li, Wise-EA MetaTrader Programming"
#property link          "www.wix.com/wiseea/wise-ea#! "
#property version       "1.04"
#property description   "Copyright Daniel Luchinger"
#property description   "Updated on 04/14/2014"
#property description   "Revised version, daily grid trading system, multiple entry methods"
//+------------------------------------------------------------------+
//| Setup & Include                                                  |
//+------------------------------------------------------------------+
#include <stdlib.mqh>
//+------------------------------------------------------------------+
//| Input parameters                                                 |
//+------------------------------------------------------------------+
input string TRAD                = " --- Trade Opening Settings --- "; 
input bool   Strategy1           = true;
input double RSIUpper1           = 70;
input double RSILower1           = 30;
input bool   Strategy2           = false;
input double RSIUpper2           = 90;
input double RSILower2           = 10;
input int    RSIPeriod           = 5;
input string StartTradingTime    = "01:00";
input string EndTradingTime      = "12:00";
input double GridInterval        = 10;
input double InitialGap          = 5;
input int    MaxPendingOrders    = 3;
input double StopLoss            = 10;
input double TakeProfit          = 20;
input double LotMultiplier       = 1.5;
input double FirstOrderLot       = 0.01;
input bool   UseAutoLots         = true;
input double Risk_Ratio          = 2;
input string MANAGE              = " --- Management & Exit Settings --- "; 
input bool   UseTrendFilter      = false;
input string CloseTime           = "15:00";
input bool   CloseAtBasePrice    = true;
input bool   UseBE               = false;
input double BEPoint             = 25;
input double BEProfitPip         = 2;
input bool   UseTrailingStop     = false;
input double TrailActivation     = 30;
input double TrailingStop        = 20;
input double TrailingStep        = 1;
input string MISC                = " --- Miscellaneous Settings --- ";
input int    MagicNumber         = 765126; 
input double Slippage            = 2; 
input string ID                  = "Timekeeper";
input bool   SendAlerts          = true;
//+------------------------------------------------------------------+
//| Global variabels                                                 |
//+------------------------------------------------------------------+
int    dig, stl, count, dir[2], stage[2], cnt[6], pendingcnt[2];
double pnt, pprice[2], pfit[2], phfit[2], plt[2], rsi, rsi1;
bool   trade[2], stra[2], close;
string EAComment, trend, rtrend;
datetime eastarttime, cycletime[2];
//+------------------------------------------------------------------+
//| EA initialization function                                       |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//---- initialize variables
   dig=SymbolInfoInteger(Symbol(),SYMBOL_DIGITS);
   pnt=SymbolInfoDouble(Symbol(),SYMBOL_POINT);
   if (dig==3 || dig==5) pnt*=10; 
   stl=SymbolInfoInteger(Symbol(),SYMBOL_TRADE_STOPS_LEVEL)*Point/pnt; 
   infosend("Broker stop level: "+stl+" pips.");
   EAComment=ID+" "+Symbol()+string(Period());
   eastarttime=TimeCurrent();
   dir[0]=-1;
   dir[1]=-1;
   trade[0]=true;
   trade[1]=true;
   stra[0]=Strategy1;
   stra[1]=Strategy2;
//----
   if (!IsOptimization() && !IsVisualMode())
     {
      if (!HLineCreate(0,ID+"HLine"+string(0),0,0,clrWhite,STYLE_SOLID,1)) return(-1);
      if (!HLineCreate(0,ID+"HLine"+string(1),0,0,clrYellow,STYLE_SOLID,1)) return(-1);
      if (!RectLabelCreate(0,ID+"RecLable",0,0,20,20,200,110)) return(-1); 
      for (int i=0; i<5; i++)
        {
         color clr=clrWhite;
         if (i==1) clr=clrAqua;
         if (i==2) clr=clrYellow;
         if (!LabelCreate(0,ID+"TextLable"+string(i),0,0,30,30+i*18,"Lable","Arial",10,clr)) return(-1); 
        }
     }     
//----
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| EA deinitialization function                                     |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- 
   Comment("");
   delete_pending(0);
   delete_pending(1);
//--- 
  }  
//+------------------------------------------------------------------+
//| OnTick function                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//----
   trend="Buy"; 
   double red=iCustom(NULL,0,"i_Trend",1,0);
   double green=iCustom(NULL,0,"i_Trend",0,0);
   if (red>green) trend="Sell";
//----   
   rtrend="Mid";    
   rsi=NormalizeDouble(iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,0),2);
   rsi1=NormalizeDouble(iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,1),2);
   if (rsi>=RSIUpper1) rtrend="Upper";
   else if (rsi<=RSILower1) rtrend="Lower";
//----
   close=false;
   datetime closetime=StrToTime(TimeToStr(TimeCurrent(),TIME_DATE)+" "+CloseTime);
   if (TimeCurrent()>=closetime && !tradetime(StartTradingTime,EndTradingTime)) close=true;
//----   
   for (int x=0; x<=1; x++)
     {
      if (!trade[x]) delete_pending(x); 
      count_trades(x);   
      if (count==0) 
        {
         stage[x]=0; trade[x]=true; pfit[x]=0; phfit[x]=0; 
         if (!IsOptimization() && !IsVisualMode()) {if (!ObjMove(0,ID+"HLine"+string(x),0)) return;}
        }
      if (stage[x]==0) 
        {
         pprice[x]=0;
         if (stra[x] && tradetime(StartTradingTime,EndTradingTime))
           {
            if ((x==0 && rtrend=="Mid") || (x==1 && lastclosebar(1)!=0 && ((rsi>=RSIUpper2 && rsi1<RSIUpper2) || (rsi<=RSILower2 && rsi1>RSILower2))))
              {
               if (!IsOptimization() && !IsVisualMode()) {if (!ObjMove(0,ID+"HLine"+string(x),Bid)) return;}
               if (cnt[4]==0) opentrade(4,0,Bid+InitialGap*pnt,x);
               if (cnt[5]==0) opentrade(5,0,Bid-InitialGap*pnt,x);
               if (cnt[4]==1 && cnt[5]==1) stage[x]=1;
              }
           }
        }
      else if (stage[x]==1)
        {
         if (cnt[0]==1) {dir[x]=0; stage[x]=2; trade[x]=true; pendingcnt[x]=0;}
         else if (cnt[1]==1) {dir[x]=1; stage[x]=2; trade[x]=true; pendingcnt[x]=0;}
         if (x==0 && rtrend!="Mid") {infosend("RSI no longer in middle zone, current cycle closed."); delete_pending(x);}   
        }
      else if (stage[x]==2) {manage_trades(x); check_history(x);}
     }
//----
   screen_display();
//----
  }  
//******************************************************************************************************
//******************************************************************************************************
//+------------------------------------------------------------------+
//| FUNCTIONS: Which bar last trade closed                           |
//+------------------------------------------------------------------+
int lastclosebar(int x)
  {
//----
   int barcnt=-1;
   for (int i=OrdersHistoryTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<eastarttime) continue;
      if (OrderType()<=1) {barcnt=iBarShift(NULL,0,OrderOpenTime()); break;}
     }
//----
   return (barcnt);
  }          
//+------------------------------------------------------------------+
//| FUNCTIONS: Count trades                                          |
//+------------------------------------------------------------------+
void count_trades(int x) 
  {
//----
   count=0;
   ArrayInitialize(cnt,0);
   for (int i=OrdersTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<eastarttime) continue;
      //----
      count++;
      cnt[OrderType()]++;
     }
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Check hisotry profit/loss                             |
//+------------------------------------------------------------------+  
void check_history(int x)
  {
//----
   phfit[x]=0;
   for (int i=OrdersHistoryTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<cycletime[x]) continue;
      if (OrderType()>1) continue;
      phfit[x]=phfit[x]+OrderProfit();
     }
  } 
//+------------------------------------------------------------------+
//| FUNCTIONS: Manage trades                                         |
//+------------------------------------------------------------------+
void manage_trades(int x) // for stage 2 
  {
//----
   if (trade[x] && UseTrendFilter && ((dir[x]==0 && trend=="Sell") || (dir[x]==1 && trend=="Buy"))) 
     {
      trade[x]=false;
      infosend("iTrend is against current cycle direction, no more orders will be opened."); 
     }
   if (x==0 && trade[x] && rtrend!="Mid") 
     {
      trade[x]=false;
      infosend("RSI no longer in middle zone, no more orders will be opened."); 
     }     
//----    
   int hiindex=0;
   pfit[x]=0;
   for (int i=0; i<OrdersTotal(); i++) 
     {
      if (!OrderSelect(i,SELECT_BY_POS)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<eastarttime) continue;
      //----
      if (close || cnt[dir[x]]==0 || (CloseAtBasePrice && pprice[x]>0 && ((dir[x]==0 && Bid<=pprice[x]-InitialGap*pnt) || (dir[x]==1 && Bid>=pprice[x]+InitialGap*pnt))))
        {
         if (OrderType()<=1) closetrade();
         else if (OrderDelete(OrderTicket())) infosend("Order ("+OrderTicket()+") has been deleted."); 
        }
      else 
        {  
         int index=StrToInteger(OrderComment());
         if (OrderType()<=1)
           {
            pfit[x]=pfit[x]+OrderProfit();
            if (pprice[x]==0) {pprice[x]=OrderOpenPrice(); cycletime[x]=OrderOpenTime(); plt[x]=OrderLots();}
            if (index>hiindex) hiindex=index;
            if (UseBE) check_breakeven();
            if (UseTrailingStop) check_trailingstop();
           }
         else if ((OrderType()==4 && dir[x]==1) || (OrderType()==5 && dir[x]==0))
           {
            if (OrderDelete(OrderTicket())) infosend("Cycle direction established, opposite order ("+OrderTicket()+") has been deleted."); 
           }
        }
     }
//----
   if (!close && cnt[dir[x]]>0 && trade[x])
     {
      for (i=0; i<=hiindex+1; i++)
        {
         if (dir[x]==0 && (pendingcnt[x]<MaxPendingOrders || pendingcnt[x]==0))
           {
            double prc=pprice[x]+i*GridInterval*pnt;
            if (gridordercnt(i,x)==0 && prc>=Ask+stl*pnt && prc>0) opentrade(4,i,prc,x);   
           }
         else if (dir[x]==1 && (pendingcnt[x]<MaxPendingOrders || pendingcnt[x]==0))
           {
            prc=pprice[x]-i*GridInterval*pnt; 
            if (gridordercnt(i,x)==0 && prc<=Bid-stl*pnt && prc>0) opentrade(5,i,prc,x);   
           }
        }
     } 
  } 
//+------------------------------------------------------------------+
//| FUNCTIONS: Count trade of a certain grid                         |
//+------------------------------------------------------------------+
int gridordercnt(int gridindex, int x) 
  {
//----
   int gridcnt=0;
   for (int i=OrdersTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<eastarttime) continue;
      //----
      int index=StrToInteger(OrderComment());
      if (dir[x]==0 && index==gridindex && (OrderType()==0 || OrderType()==4)) {gridcnt++; break;}
      else if (dir[x]==1 && index==gridindex && (OrderType()==1 || OrderType()==5)) {gridcnt++; break;}
     }
//----
   return (gridcnt);     
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Trailing stop                                         |
//+------------------------------------------------------------------+  
void check_trailingstop() 
  {
//----
   if (OrderType()==0 && Bid>=OrderOpenPrice()+TrailActivation*pnt) 
     {
      if (OrderStopLoss()==0 || OrderStopLoss()<Bid-TrailingStop*pnt-TrailingStep*pnt)    
         if (OrderModify(OrderTicket(),OrderOpenPrice(),Bid-TrailingStop*pnt,OrderTakeProfit(),0,Lime)) return;
     } 
   else if (OrderType()==1 && Ask<=OrderOpenPrice()-TrailActivation*pnt) 
     {
      if (OrderStopLoss()==0 || OrderStopLoss()>Ask+TrailingStop*pnt+TrailingStep*pnt) 
         if (OrderModify(OrderTicket(),OrderOpenPrice(),Ask+TrailingStop*pnt,OrderTakeProfit(),0,Red)) return;
     }
  }   
//+------------------------------------------------------------------+
//| FUNCTIONS: Check break even                                      |
//+------------------------------------------------------------------+  
void check_breakeven() 
  {
//----
   if (OrderType()==0) 
     {
      double sl=NormalizeDouble(OrderOpenPrice()+BEProfitPip*pnt,dig);
      if ((OrderStopLoss()==0 || NormalizeDouble(OrderStopLoss(),dig)<sl) && (Bid>=OrderOpenPrice()+BEPoint*pnt)) 
        { 
         bool result=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,CLR_NONE);
         if (result) infosend("Order ("+OrderTicket()+") set to BreakEven : "+" @"+DoubleToStr(sl,dig)); 
         else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+"Error set to BreakEven : "+ErrorDescription(GetLastError()));
        }
     }          
   else if (OrderType()==1) 
     {
      sl=NormalizeDouble(OrderOpenPrice()-BEProfitPip*pnt,dig);
      if ((OrderStopLoss()==0 || NormalizeDouble(OrderStopLoss(),dig)>sl) && (Ask<=OrderOpenPrice()-BEPoint*pnt)) 
        { 
         result=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0,CLR_NONE);
         if (result) infosend("Order ("+OrderTicket()+") set to BreakEven : "+" @"+DoubleToStr(sl,dig));
         else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+"Error set to BreakEven : "+ErrorDescription(GetLastError()));
        }
     }   
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Delete pending orders                                 |
//+------------------------------------------------------------------+
void delete_pending(int x) 
  {
//----
   for (int i=OrdersTotal()-1; i>=0; i--) 
     {
      if (!OrderSelect(i,SELECT_BY_POS)) continue;
      if (OrderSymbol()!=Symbol()) continue;
      if (OrderMagicNumber()!=MagicNumber+x) continue;
      if (OrderOpenTime()<eastarttime) continue;
      //----
      if (OrderType()>1 && OrderDelete(OrderTicket())) infosend("Order ("+OrderTicket()+") has been deleted."); 
     }
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Trading times                                         |
//+------------------------------------------------------------------+
bool tradetime(string startstr, string endstr) 
  {
//----
   datetime starttime=StrToTime(TimeToStr(TimeCurrent(),TIME_DATE)+" "+startstr);
   datetime endtime=StrToTime(TimeToStr(TimeCurrent(),TIME_DATE)+" "+endstr);
//----   
   bool time=false;
   if (starttime<endtime) 
     {
      if (TimeCurrent()>=starttime && TimeCurrent()<endtime) time=true; 
     }
   else if (starttime>endtime) 
     {
      if (TimeCurrent()>=starttime || TimeCurrent()<endtime) time=true;
     }
   else if (starttime==endtime) time=true; 
   if (startstr=="" || endstr=="") time=true; 
//---- 
   return (time);
  }
//******************************************************************************************************
//******************************************************************************************************  
//+------------------------------------------------------------------+
//| FUNCTIONS: Order excution                                        |
//+------------------------------------------------------------------+
void opentrade(int ordertype, int index, double price, int x) 
  {
//----
   string typestring; 
   double stoploss,takeprofit,lt;
   color  clr=Lime;
   if (ordertype==1 || ordertype==3 || ordertype==5) clr=Red;
//----
   //Alert("Orderprice: ",price);
   if (ordertype==4) typestring="Strategy "+string(x+1)+" BUY STOP";
   else if (ordertype==5) typestring="Strategy "+string(x+1)+" SELL STOP";
   if (index==0) lt=LotsOptimized();
   else lt=normalizelot(plt[x]*MathPow(LotMultiplier,index));
//----  
   int ticket=OrderSend(Symbol(),ordertype,lt,price,Slippage,0,0,string(index),MagicNumber+x,0,clr);
   if(ticket>0 && OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
     {
      infosend(typestring+" order ("+OrderTicket()+") opened: "+" @"+DoubleToStr(OrderOpenPrice(),dig));
      cnt[ordertype]++;
      if (OrderType()>1 && stage[x]==2) pendingcnt[x]++;
      //----
      if (ordertype==4) 
        {
         if (StopLoss!=0) stoploss=OrderOpenPrice()-StopLoss*pnt;
         else stoploss=0;
         if (TakeProfit!=0) takeprofit=OrderOpenPrice()+TakeProfit*pnt;
         else takeprofit=0;
        }
      else if (ordertype==5) 
        {
         if (StopLoss!=0) stoploss=OrderOpenPrice()+StopLoss*pnt;
         else stoploss=0;
         if (TakeProfit!=0) takeprofit=OrderOpenPrice()-TakeProfit*pnt;
         else takeprofit=0;
        } 
      if (takeprofit!=0 || stoploss!=0) 
        {     
         for (int j=10; j>0; j--) 
           {
            if (OrderModify(ticket,OrderOpenPrice(),stoploss,takeprofit,0,CLR_NONE)) break;
            else Sleep(1000);  
            RefreshRates();
           }
        } 
     }
   else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Error opening order : ",ErrorDescription(GetLastError())); 
  } 
//+------------------------------------------------------------------+
//| FUNCTION: Trade close                                            |
//+------------------------------------------------------------------+  
void closetrade()  
  {
//----
   double price; 
   if (OrderType()==0) price=Bid;
   else if (OrderType()==1) price=Ask;      
   bool result=OrderClose(OrderTicket(),OrderLots(),price,Slippage,MediumSeaGreen);
   if (result) infosend("Order ("+OrderTicket()+")  closed: "+" @"+DoubleToStr(OrderClosePrice(),dig));
   else Print(TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" | "+" Error closing order : ",ErrorDescription(GetLastError()));
  }   
//+------------------------------------------------------------------+
//| FUNCTIONS: Money management                                      |
//+------------------------------------------------------------------+
double LotsOptimized() 
  {
//----
   double step=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
   double minlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   double maxlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
//----
   double val=0;
   if (UseAutoLots) 
     {   
      if (step==0.01) int digit=2;
      else if (step==0.1) digit=1;
      else digit=0;
      //----
      double maxloss=AccountInfoDouble(ACCOUNT_BALANCE)*Risk_Ratio/100;   
      double pointcost=SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_VALUE);
      double stoppoint=MathAbs(StopLoss*pnt/Point);
      if (stoppoint*pointcost>0) val=maxloss/(stoppoint*pointcost); 
      double nval=NormalizeDouble(val,digit);
      if (nval<=val) val=nval;
      else val=nval-step;
     }
   else val=FirstOrderLot;
//---- 
   if (val<minlot) val=minlot;
   else if (val>maxlot) val=maxlot;
   return(val);  
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Calculate valid lot size                              |
//+------------------------------------------------------------------+
double normalizelot(double size) {
//----
   double step=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
   double minlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   double maxlot=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
//----
   if (step==0.01) int digit=2;
   else if (step==0.1) digit=1;
   else digit=0;
   double val=size;
   double nval=NormalizeDouble(val,digit);
   if (nval<=val) val=nval;
   else val=nval-step;
//----
   if (val<minlot) val=minlot;
   else if (val>maxlot) val=maxlot;
   return(val);  
}     
//******************************************************************************************************
//******************************************************************************************************
//+------------------------------------------------------------------+
//| FUNCTIONS: Debug message                                         |
//+------------------------------------------------------------------+
void infosend(string str)
  {
//----
   string body=TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS)+" | "+EAComment+" |  "+str;
   if (SendAlerts) Alert(body); 
   else Print(body);
  }
//+------------------------------------------------------------------+
//| FUNCTION: Create the horizontal line                             |
//+------------------------------------------------------------------+
//HLineCreate(0,ID+"HLine",0,Close[3],clrRed,STYLE_SOLID,1)
bool HLineCreate(const long            chart_ID=0,        // chart's ID
                 const string          name="HLine",      // line name
                 const int             sub_window=0,      // subwindow index
                 double                price=0,           // line price
                 const color           clr=clrRed,        // line color
                 const ENUM_LINE_STYLE style=STYLE_SOLID, // line style
                 const int             width=1,           // line width
                 const bool            back=false,        // in the background
                 const bool            selection=true,    // highlight to move
                 const bool            hidden=true,       // hidden in the object list
                 const long            z_order=0)         // priority for mouse click
  {
//--- create a horizontal line
   if(!ObjectCreate(chart_ID,name,OBJ_HLINE,sub_window,0,price))
     {
      Print(__FUNCTION__,
            ": failed to create a horizontal line! Error code = ",GetLastError());
      return(false);
     }
//--- 
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,width);
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| FUNCTION: Move object with 1 price point                         |
//+------------------------------------------------------------------+
bool ObjMove(const long   chart_ID=0,   // chart's ID
             const string name="HLine", // line name
             double       price=0)      // line price
  {
//--- move a horizontal line
   if(!ObjectMove(chart_ID,name,0,0,price))
     {
      Print(__FUNCTION__,
            ": failed to move the object! Error code = ",GetLastError());
      return(false);
     }
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| FUNCTION: Create rectangle label                                 |
//+------------------------------------------------------------------+
//RectLabelCreate(0,ID+"RecLable",0,ChartCorner,20,20,50,50,clrSilver,BORDER_RAISED)
bool RectLabelCreate(const long             chart_ID=0,               // chart's ID
                     const string           name="RectLabel",         // label name
                     const int              sub_window=0,             // subwindow index
                     const ENUM_BASE_CORNER corner=CORNER_LEFT_UPPER, // chart corner for anchoring
                     const int              x=20,                     // X coordinate
                     const int              y=20,                     // Y coordinate
                     const int              width=60,                 // width
                     const int              height=60,                // height
                     const color            back_clr=C'0x35,0x35,0x35', // background color
                     const ENUM_BORDER_TYPE border=BORDER_FLAT,       // border type
                     const color            clr=clrWhiteSmoke,        // flat border color (Flat)
                     const ENUM_LINE_STYLE  style=STYLE_SOLID,        // flat border style
                     const int              line_width=1,             // flat border width
                     const bool             back=false,               // in the background
                     const bool             selection=false,          // highlight to move
                     const bool             hidden=true,              // hidden in the object list
                     const long             z_order=0)                // priority for mouse click
  {
//--- create a rectangle label
   if(!ObjectCreate(chart_ID,name,OBJ_RECTANGLE_LABEL,sub_window,0,0))
     {
      Print(__FUNCTION__,
            ": failed to create a rectangle label! Error code = ",GetLastError());
      return(false);
     }
//--- 
   ObjectSetInteger(chart_ID,name,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(chart_ID,name,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(chart_ID,name,OBJPROP_XSIZE,width);
   ObjectSetInteger(chart_ID,name,OBJPROP_YSIZE,height);
   ObjectSetInteger(chart_ID,name,OBJPROP_BGCOLOR,back_clr);
   ObjectSetInteger(chart_ID,name,OBJPROP_BORDER_TYPE,border);
   ObjectSetInteger(chart_ID,name,OBJPROP_CORNER,corner);
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,line_width);
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| FUNCTION: Create text lable object                               |
//+------------------------------------------------------------------+
//LabelCreate(0,ID+"TextLable",0,ChartCorner,20,20,"Lable","Arial",10,clrRed,ANCHOR_UPPER)
bool LabelCreate(const long              chart_ID=0,               // chart's ID
                 const string            name="TextLabel",         // label name
                 const int               sub_window=0,             // subwindow index
                 const ENUM_BASE_CORNER  corner=CORNER_LEFT_UPPER, // chart corner for anchoring
                 const int               x=20,                     // X coordinate
                 const int               y=20,                     // Y coordinate
                 const string            text="TextLabel",         // text
                 const string            font="Arial",             // font
                 const int               font_size=10,             // font size
                 const color             clr=clrWhite,             // color
                 const ENUM_ANCHOR_POINT anchor=ANCHOR_LEFT_UPPER, // anchor type
                 const double            angle=0.0,                // text slope
                 const bool              back=false,               // in the background
                 const bool              selection=false,          // highlight to move
                 const bool              hidden=true,              // hidden in the object list
                 const long              z_order=0)                // priority for mouse click
  {
//--- create a text label
   if(!ObjectCreate(chart_ID,name,OBJ_LABEL,sub_window,0,0))
     {
      Print(__FUNCTION__,
            ": failed to create text label! Error code = ",GetLastError());
      return(false);
     }
//--- 
   ObjectSetInteger(chart_ID,name,OBJPROP_XDISTANCE,x);
   ObjectSetInteger(chart_ID,name,OBJPROP_YDISTANCE,y);
   ObjectSetInteger(chart_ID,name,OBJPROP_CORNER,corner);
   ObjectSetString(chart_ID,name,OBJPROP_TEXT,text);
   ObjectSetString(chart_ID,name,OBJPROP_FONT,font);
   ObjectSetInteger(chart_ID,name,OBJPROP_FONTSIZE,font_size);
   ObjectSetDouble(chart_ID,name,OBJPROP_ANGLE,angle);
   ObjectSetInteger(chart_ID,name,OBJPROP_ANCHOR,anchor);
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  } 
//+------------------------------------------------------------------+
//| FUNCTION: Bool to string                                         |
//+------------------------------------------------------------------+
string BoolToStr(bool bval)   
  {
//----
   if (bval) return("True");
   return("False");
  }
//+------------------------------------------------------------------+
//| FUNCTIONS: Screen display                                        |
//+------------------------------------------------------------------+
void screen_display() 
  {
//----
   color clr=clrWhite;
   ObjectSetString(0,ID+"TextLable"+string(0),OBJPROP_TEXT,"Trading Status: "+BoolToStr(tradetime(StartTradingTime,EndTradingTime)));
   ObjectSetString(0,ID+"TextLable"+string(1),OBJPROP_TEXT,"Account Balance: "+DoubleToStr(AccountBalance(),2));
   ObjectSetString(0,ID+"TextLable"+string(2),OBJPROP_TEXT,"Account Equity: "+DoubleToStr(AccountEquity(),2));
   ObjectSetString(0,ID+"TextLable"+string(3),OBJPROP_TEXT,"Open Profit/Loss: "+DoubleToStr(pfit[0],2)+"/"+DoubleToStr(pfit[1],2));
   if (pfit[0]+pfit[1]>0) clr=clrLimeGreen; 
   else if (pfit[0]+pfit[1]<0) clr=clrRed;   
   ObjectSetInteger(0,ID+"TextLable"+string(3),OBJPROP_COLOR,clr);
   ObjectSetString(0,ID+"TextLable"+string(4),OBJPROP_TEXT,"Closed Profit/Loss: "+DoubleToStr(phfit[0],2)+"/"+DoubleToStr(phfit[1],2));
   if (phfit[0]+phfit[1]>0) clr=clrLimeGreen;
   else if (phfit[0]+phfit[1]<0) clr=clrRed;
   ObjectSetInteger(0,ID+"TextLable"+string(4),OBJPROP_COLOR,clr);
  }   